@@ -5,6 +5,8 @@
 #'@param filename String with the filename. If `.obj` is not at the end of the string, it will be appended automatically.
 #'@param save_texture Default `TRUE`. If the texture should be saved along with the geometry.
 #'@param water_index_refraction Default `1`. The index of refraction for the rendered water.
+#'@param manifold_geometry Default `FALSE`. If `TRUE`, this will take the additional step of making the mesh manifold.
+#'@param all_face_fields Default `FALSE`. If `TRUE`, all OBJ face fields (v/vn/vt) will always be written.
 #'@export
 #'@examples
 #'filename_obj = tempfile(fileext = ".obj")
@@ -33,7 +35,8 @@
 #'save_obj(filename_obj, water_index_refraction = 1.5)
 #'rgl::rgl.close()
 #'}
-save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
+save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1, 
+                    manifold_geometry = FALSE, all_face_fields = FALSE) {
   if(is.null(filename)) {
     stop("save_obj requires a filename")
   }
@@ -81,6 +84,7 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
     # allvertices <<- rbind(allvertices,vertices) #Debug line
     textures = rgl.attrib(id, "texcoords")
     normals = rgl.attrib(id, "normals")
+    # has_degenerate_normals = any(normals[,1] == 0 & normals[,2] == 0 & normals[,3] == 0)
     cat(paste0("v ", sprintf("%1.6f %1.6f %1.6f",vertices[,1], vertices[,2], vertices[,3])), file=con, sep = "\n")
     number_vertices <<- number_vertices + nrow(vertices)
     if (nrow(textures) > 0) {
@@ -88,6 +92,7 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
       number_texcoords <<- number_texcoords + nrow(textures)
     }
     if (nrow(normals) > 0) {
+      
       cat(paste0("vn ",sprintf("%1.6f %1.6f %1.6f", normals[,1], normals[,2], normals[,3])), file=con, sep = "\n" )
       number_normals <<- number_normals + nrow(normals)
     }
@@ -150,8 +155,17 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
       cat(paste("Kd", sprintf("%1.4f %1.4f %1.4f",tempcol[1],tempcol[2],tempcol[3]),collapse = " "), file=con, sep="\n")
       cat(paste("Ka", sprintf("%1.4f %1.4f %1.4f",tempcol[1],tempcol[2],tempcol[3]),collapse = " "), file=con, sep="\n")
       cat("\n", file=con)
+    } else if (!is.na(idrow$tricolor[[1]])) {
+      tempcol = col2rgb(idrow$tricolor[[1]])/256
+      cat(paste("newmtl ray_polygon3d"), file=con, sep="\n")
+      cat(paste("Kd", sprintf("%1.4f %1.4f %1.4f",tempcol[1],tempcol[2],tempcol[3]),collapse = " "), file=con, sep="\n")
+      cat(paste("Ka", sprintf("%1.4f %1.4f %1.4f",tempcol[1],tempcol[2],tempcol[3]),collapse = " "), file=con, sep="\n")
+      cat("\n", file=con)
     }
   }
+  write_comment = function(comment, con) {
+    cat(comment, file=con)
+  }
   
   string_num = function(n) {
     sprintf("%d", n)
@@ -171,20 +185,28 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
   vertex_info$endindextex = NA
   vertex_info$endindexnormals = NA
   wrote_water = FALSE
-  
+  wrote_base = FALSE
   for(row in 1:nrow(vertex_info)) {
     id = vertex_info$id[row]
     if(vertex_info$raytype[row] %in% c("surface","base","basebottom","water",
                                        "north_symbol","arrow_symbol", "bevel_symbol",
                                        "background_symbol", "scalebar_col1", "scalebar_col2",
-                                       "surface_tris")) {
+                                       "surface_tris","polygon3d")) {
       vertex_info$startindex[row] = number_vertices + 1
       vertex_info$startindextex[row] = number_texcoords + 1
       vertex_info$startindexnormals[row] = number_normals + 1
+      write_comment(sprintf("# %s\n",vertex_info$raytype[row]), con)
       write_data(id, con)
       if(vertex_info$raytype[row] != "water") {
         if(save_texture) {
-          write_mtl(vertex_info[row,], con_mtl)
+          if(vertex_info$raytype[row] != "base") {
+            write_mtl(vertex_info[row,], con_mtl)
+          } else {
+            if(!wrote_base) {
+              write_mtl(vertex_info[row,], con_mtl)
+              wrote_base = TRUE
+            }
+          }
         }
       } else {
         if(!wrote_water) {
@@ -233,7 +255,7 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
       if(hasnormals) {
         indices = sprintf("%d/%d/%d", indices, indices, indices)
       } else {
-        indices = sprintf("%d/%d", indices, indices)
+        indices = sprintf("%d/%d/", indices, indices)
       }
       indices = matrix(indices, ncol=3, byrow=TRUE)
       indices = indices[1:(nrow(indices)-na_counter),]
@@ -250,7 +272,11 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
         hasnormals = FALSE
       }
       indices = vertex_info$startindextex[row]:vertex_info$endindex[row]
-      indices = sprintf("%d/%d", indices, indices)
+      if(hasnormals) {
+        indices = sprintf("%d/%d/%d", indices, indices, indices)
+      } else {
+        indices = sprintf("%d/%d/", indices, indices)
+      }
       indices = matrix(indices, ncol=3, byrow=TRUE)
       cat(paste("f", indices[,1], indices[,2], indices[,3]), 
           sep="\n", file=con)
@@ -259,6 +285,11 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
         cat("g Basebottom", file=con, sep ="\n")
         cat("usemtl ray_base", file=con, sep ="\n")
       }
+      if(!is.na(vertex_info$startindexnormals[row]) && !is.na(vertex_info$endindexnormals[row])) {
+        hasnormals = vertex_info$startindexnormals[row] < vertex_info$endindexnormals[row]
+      } else {
+        hasnormals = FALSE
+      }
       dims = rgl::rgl.attrib(vertex_info$id[row], "dim")
       vertices_y = rgl.attrib(vertex_info$id[row], "vertices")[,2]
       nx = dims[1]
@@ -280,7 +311,12 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
         }
       }
       indices = indices + vertex_info$startindextex[row] - 1
-      indices = sprintf("%d//%d", indices, indices)
+      if(all_face_fields) {
+        indices = sprintf("%d//", indices)
+      } else {
+        indices = sprintf("%d", indices)
+      }
+      
       indices = matrix(indices, ncol=3, byrow=TRUE)
       indices = indices[1:(nrow(indices)-na_counter),]
       cat(paste("f", indices[,3], indices[,2], indices[,1]), 
@@ -296,10 +332,16 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
         hasnormals = FALSE
       }
       baseindices = matrix(vertex_info$startindex[row]:vertex_info$endindex[row], ncol=3, byrow=TRUE)
-      if(hasnormals) {
-        cat(paste("f", sprintf("%d//%d %d//%d %d//%d", baseindices[,1], baseindices[,1],
-                               baseindices[,2], baseindices[,2], 
-                               baseindices[,3], baseindices[,3])), 
+      # if(hasnormals) {
+      #   cat(paste("f", sprintf("%d//%d %d//%d %d//%d", baseindices[,1], baseindices[,1],
+      #                          baseindices[,2], baseindices[,2], 
+      #                          baseindices[,3], baseindices[,3])), 
+            # sep="\n", file=con)
+      # } else {
+      if(all_face_fields) {
+        cat(paste("f", sprintf("%d// %d// %d//", baseindices[,1],
+                               baseindices[,2], 
+                               baseindices[,3])), 
             sep="\n", file=con)
       } else {
         cat(paste("f", sprintf("%d %d %d", baseindices[,1],
@@ -307,6 +349,7 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
                                baseindices[,3])), 
             sep="\n", file=con)
       }
+      # }
     } else if (vertex_info$raytype[row] == "water") {
       if(save_texture) {
         cat("g Water", file=con, sep ="\n")
@@ -404,6 +447,17 @@ save_obj = function(filename, save_texture = TRUE, water_index_refraction = 1) {
         cat(paste("f", baseindices[,1], baseindices[,2], baseindices[,3]), 
             sep="\n", file=con)
       }
+    } else if (vertex_info$raytype[row] == "polygon3d") {
+      if(save_texture) {
+        cat("g Polygon", file=con, sep ="\n")
+        cat("usemtl ray_polygon3d", file=con, sep ="\n")
+      }
+      baseindices = matrix(vertex_info$startindex[row]:vertex_info$endindex[row], ncol=3, byrow=TRUE)
+      cat(paste("f", baseindices[,1], baseindices[,2], baseindices[,3]), 
+          sep="\n", file=con)
     }
   }
+  if(manifold_geometry) {
+    fix_manifold_geometry(filename)
+  }
 }
